@@ -276,17 +276,25 @@ def send_block_task_states_to_caller_task(func):
     return wrapper
 
 
-def wait_for_running_tasks_from_results(results, max_wait=3*60, sleep_between_iterations=0.05):
-    run_states = set(READY_STATES)
-    run_states.add(STARTED)
+def wait_for_running_tasks_from_results(results, max_wait=2*60, sleep_between_iterations=0.05):
+    run_states = set(READY_STATES) | {STARTED, RETRY}
     running_tasks = []
     for result in results:
         if result.state in run_states and not get_task_postrun_info(result):
             running_tasks.append(result)
 
     max_sleep = sleep_between_iterations * 20  # Somewhat arbitrary
-    start_time = time.monotonic()
-    while running_tasks and (not max_wait or (time.monotonic() - start_time < max_wait)):
+    start_time = last_debug_output = time.monotonic()
+    while running_tasks:
+        time_now = time.monotonic()
+
+        if time_now - last_debug_output >= 30:
+            logger.debug(f'Waiting for running task(s): {get_tasks_names_from_results(running_tasks)}')
+            last_debug_output = time_now
+
+        if max_wait and time_now - start_time >= max_wait:
+            break
+
         time.sleep(sleep_between_iterations)
         running_tasks = [result for result in running_tasks if not get_task_postrun_info(result)]
         sleep_between_iterations = sleep_between_iterations * 1.01 \
@@ -304,8 +312,8 @@ def wait_on_async_results(results,
                           sleep_between_iterations=0.05,
                           check_task_worker_frequency=600,
                           fail_on_worker_failures=7,
-                          log_msg=True
-                          ):
+                          log_msg=True,
+                          **_kwargs):
     if not results:
         return
 
@@ -359,16 +367,18 @@ def wait_on_async_results(results,
                 sleep_between_iterations = sleep_between_iterations * 1.01 \
                     if sleep_between_iterations*1.01 < max_sleep else max_sleep  # Exponential backoff
 
-            if result.state == REVOKED:
+            result_state = result.state
+            # Revoked tasks now go into retry sometimes in new Celery 5.1.0.
+            if result_state == REVOKED or result_state == RETRY:
                 #  wait for revoked tasks to actually finish running
                 wait_for_running_tasks_from_results([result])
                 raise ChainRevokedException(task_id=str(result),
                                             task_name=get_task_name_from_result(result))
-            if result.state == PENDING:
+            if result_state == PENDING:
                 # Pending tasks can be in revoke list. State will still be PENDING.
                 raise ChainRevokedPreRunException(task_id=str(result),
                                                   task_name=get_task_name_from_result(result))
-            if result.state == FAILURE:
+            if result_state == FAILURE:
                 cause = result.result if isinstance(result.result, Exception) else None
                 raise ChainInterruptedException(task_id=str(result),
                                                 task_name=get_task_name_from_result(result),
